package hr.fer.ga.imgcompression;
/**
 * 
 */
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.jgap.Chromosome;
import org.jgap.Configuration;
import org.jgap.Gene;
import org.jgap.Genotype;
import org.jgap.IChromosome;
import org.jgap.IGeneticOperatorConstraint;
import org.jgap.IUniversalRateCalculator;
import org.jgap.InvalidConfigurationException;
import org.jgap.Population;
import org.jgap.RandomGenerator;
import org.jgap.impl.CrossoverOperator;


/**
 *
 */
public class CrossoverOperator2 extends CrossoverOperator {

	/**
	 * vjerojatnost krianja zadana brojem jedinki koje
	 * e se kriati (iskljuivo je s m_crossoverRatePrecent i m_crossoverRateCalc)
	 */
	private int m_crossoverRate;
	
	/**
	 * vjerojatnost krianja zadana postotkom jedinki populacije
	 * (iskljuivo je s m_crossoverRatePrecent i m_crossoverRateCalc)
	 */
	private double m_crossoverRatePercent;

	/**
	 * kalkulator za dinamiko odreivanje vjerojatnosti krianja
	 * (iskljuivo je s m_crossoverRatePrecent i m_crossoverRateCalc)
	 */
	private IUniversalRateCalculator m_crossoverRateCalc;
	
	/**
	 * ako je vrijednost postavljena na true kria i novo stvorene kromosome iz
	 * populacije potomaka (kromosome stare 0 godina 
	 */
	private boolean m_xoverNewAge;
	
	public CrossoverOperator2()
		throws InvalidConfigurationException {
		super(Genotype.getStaticConfiguration());
		init();
	}
	
	public CrossoverOperator2(final Configuration config) throws InvalidConfigurationException {
		super(config);
		init();
	}
		
	@Override
	protected void init() {
	    m_crossoverRate = 6;
	    m_crossoverRatePercent = -1;
	    setCrossoverRateCalc(null);
	    setXoverNewAge(true);
	}
	
	public CrossoverOperator2(final Configuration a_configuration,
				final IUniversalRateCalculator a_crossoverRateCalculator)
										throws InvalidConfigurationException {
		super(a_configuration);
		setCrossoverRateCalc(a_crossoverRateCalculator);
	}
	
	/**
	 * Postavlja kalkulator za dinamiko odreivanje vjerojatnosti krianja
	 * 
	 * @param a_crossoverRateCalculator
	 */
	private void setCrossoverRateCalc(final IUniversalRateCalculator
											a_crossoverRateCalculator) {
		m_crossoverRateCalc = a_crossoverRateCalculator;
		if (a_crossoverRateCalculator != null) {
			m_crossoverRate = -1;
			m_crossoverRatePercent = -1d;
		}
	}
	
	public CrossoverOperator2(final Configuration a_configuration,
			final double a_crossoverRatePercentage,
			final boolean a_xoverNewAge)
				throws InvalidConfigurationException {
		super(a_configuration);
		
		if (a_crossoverRatePercentage <= 0.0d) {
			throw new IllegalArgumentException("Crossover rate must be greater zero");
		}
		m_crossoverRatePercent = a_crossoverRatePercentage;
		m_crossoverRate = -1;
		setCrossoverRateCalc(null);
		setXoverNewAge(a_xoverNewAge);
	}
	
	/**
	 * Vraa gene kromosoma pospremljene u ArrayListu
	 * 
	 * @param chrom
	 * @return
	 */
	private List<PixelGene> getGeneList(IChromosome chrom) {
		List<PixelGene> geneList = new ArrayList<PixelGene>();
		Gene[] genes = chrom.getGenes();
		for(int i = 0; i < genes.length; i++)
			geneList.add((PixelGene)genes[i]);
			
		return geneList;
	}

	
	public IChromosome cross(IChromosome parent1, IChromosome parent2)
						throws InvalidConfigurationException {
		System.out.print("Kriam kromosome (operator2) " 
				+ parent1.getFitnessValue() + " i " + parent2.getFitnessValue() + " ...");
		Configuration config = getConfiguration();
		List<PixelGene> childGenes = new ArrayList<PixelGene>();
		PixelGene currentGene = null;
		int nextGeneIndex;
		List<PixelGene> parent1Genes = getGeneList(parent1);
		List<PixelGene> parent2Genes = getGeneList(parent2);
		
		
		/* dodaj prvi gen u dijete */
		if(getConfiguration().getRandomGenerator().nextBoolean()) {
			currentGene = parent1Genes.get(0);
			childGenes.add(currentGene);
		} else {
			currentGene = parent2Genes.get(0);
			childGenes.add(parent2Genes.get(0));
		}
		//System.out.println("firstGene: " + currentGene);
		
		int geneIndex;
		int parent1geneIndex;
		int parent2geneIndex;
		int distance;
		int minDistance;
		List<PixelGene> currentParent;
		
		while(parent1Genes.size() > 1) {
			nextGeneIndex = 0;
			minDistance = Integer.MAX_VALUE;
			
			currentParent = null;
			parent1geneIndex = parent1Genes.indexOf(currentGene);
			//System.out.println("parent 1 gene index: " + parent1geneIndex);
			geneIndex = parent1geneIndex - 1;		
			if(geneIndex >= 0 && geneIndex < parent1Genes.size()) {
				distance = currentGene.distance(parent1Genes.get(geneIndex));
				//System.out.println("case1");
				if(distance < minDistance) {
					minDistance = distance;
					nextGeneIndex = geneIndex;
					currentParent = parent1Genes;
					//System.out.println("choice1");
				}
			}
			
			geneIndex = parent1geneIndex + 1;
			if(geneIndex >= 0 && geneIndex < parent1Genes.size()) {
				distance = currentGene.distance(parent1Genes.get(geneIndex));
				//System.out.println("case2");
				if(distance < minDistance) {
					minDistance = distance;
					nextGeneIndex = geneIndex;
					currentParent = parent1Genes;
					//System.out.println("choice2");
				}
			}
					
			parent2geneIndex = parent2Genes.indexOf(currentGene);
			//System.out.println("parent2geneIndex: " + parent2geneIndex);
			geneIndex = parent2geneIndex - 1;		
			if(geneIndex >= 0 && geneIndex < parent2Genes.size()) {
				distance = currentGene.distance(parent2Genes.get(geneIndex));
				//System.out.println("case3");
				if(distance < minDistance) {
					minDistance = distance;
					nextGeneIndex = geneIndex;
					currentParent = parent2Genes;
					//System.out.println("choice3");
				}
			}
			
			geneIndex = parent2geneIndex + 1;
			if(geneIndex >= 0 && geneIndex < parent2Genes.size()) {
				distance = currentGene.distance(parent2Genes.get(geneIndex));
				//System.out.println("case4");
				if(distance < minDistance) {
					minDistance = distance;
					nextGeneIndex = geneIndex;
					currentParent = parent2Genes;
					//System.out.println("choice4");
				}
			}
			
			if(currentParent == null) 
				throw new IllegalStateException();
			
			currentGene = currentParent.get(nextGeneIndex);
			childGenes.add(currentGene);
			//System.out.println("pass");
			//System.out.println("currentGene: " + currentGene);
			parent1Genes.remove(parent1geneIndex);
			parent2Genes.remove(parent2geneIndex);
			
		}
		
		PixelGene[] genes =  new PixelGene[childGenes.size()];
		genes = childGenes.toArray(genes);
		System.out.println("gotov");
		return new Chromosome(config, genes);
	}
	
	/**
	 * Metoda vraa broj krianja koji je potrebno obaviti
	 * prema trenutnim postavkama.
	 * 
	 * @param populationSize
	 * @return
	 */
	private int getNumberOfCrossovers(int size) {
	    int numCrossovers = 0;
	    if (m_crossoverRate >= 0) {
	    	numCrossovers = size / m_crossoverRate;
	    } else if (m_crossoverRateCalc != null) {
	    	numCrossovers = size / m_crossoverRateCalc.calculateCurrentRate();
	    } else {
	    	numCrossovers = (int) (size * m_crossoverRatePercent);
	    }
	    return numCrossovers;
	}
	
	/**
	 * Ova metoda zove se jednom po evoluciji: mora evoluirat sve
	 * 
	 * kod evolucije: prvo zove Operatore krianja (genetic operators)
	 * onda zove prirodne selektore
	 * 
	 * @param	population 	stara populacija
	 * @param	chromosomes		izgenerirani potomci se stavljaju tu
	 */
	@Override
	public void operate(Population population, List chromosomes) {
	    int size = Math.min(getConfiguration().getPopulationSize(),
                population.size());
		int numCrossovers = getNumberOfCrossovers(population.size());
	    RandomGenerator generator = getConfiguration().getRandomGenerator();
	    IGeneticOperatorConstraint constraint = getConfiguration().
	        getJGAPFactory().getGeneticOperatorConstraint();
	    
	    int index1, index2;
	    for (int i = 0; i < numCrossovers; i++) {
	    	index1 = generator.nextInt(size);
	    	index2 = generator.nextInt(size);
	    	IChromosome parent1 = population.getChromosome(index1);
	    	IChromosome parent2 = population.getChromosome(index2);

	    	if (!isXoverNewAge() && parent1.getAge() < 1 && parent2.getAge() < 1) {
	    		continue;
	    	}
	    	if (constraint != null) {
	    		List v = new Vector();
	    		v.add(parent1);
	    		v.add(parent2);
	        	if (!constraint.isValid(population, v, this)) {
	        		/* zabranjeno je ovo krianje */
	        		continue;
	        	}
	    	}
	    	
	    	IChromosome child = null;
	    	try {
	    		child = cross(parent1, parent2);
	    	} catch(InvalidConfigurationException e) {
	    		throw new IllegalStateException();
	    	}
	    	chromosomes.add(child);
	    }
	}
}
