package hr.fer.ga.imgcompression;
/**
 * 
 */
import org.jgap.FitnessFunction;
import org.jgap.IChromosome;


/**
 *
 */
public class ImageFitnessFunction extends FitnessFunction {
	
	/** za serijalizaciju */
	private static final long serialVersionUID = 1L;
	
	/** minimalna vrijednost koju fitness funkcija moe poprimiti; odgovara najloijem moguem rjeenju */
	private double f_min = 0;
	
	/** maksimalna vrijednost koju fitness funkcija moe poprimiti; odgovara najboljem moguem rjeenju */
	private double f_max = 1.0;

	/**
	 * Metoda vraa vrijednost konstante Cmax.
	 * Cmax predstavlja maksimalnu vrijdnost F(X) - tj. vrijednost F(X)
	 * najgore mogue jedinke u populaciji.
	 * 
	 * Prema definiciji:<br>
	 * <code>Cmax = (N-1)*sum[j = 1, p](255^2)</code><br>
	 * gdje je p=4 (usporeuju se 4 bajta, ARGB)
	 * 
	 * 
	 * @param chromosome
	 * @return
	 */
	public static long getCMax(IChromosome chromosome) {
		//System.out.println("Cmax: " + (chromosome.size()-1) * 4 * 255 * 255l);
		return (chromosome.size()-1) * 4 * 255 * 255l;
	}
	
	public static long getCMaxPixel(int n) {
		return n * 255 * 255l;
	}
	
	/**
	 * Metdoa vraa vrijednost funkcije F(X).
	 * 
	 * @param chromosome
	 * @return
	 */
	public static long costFunction(IChromosome chromosome) {
		int n = chromosome.size();
		long result = 0;
		PixelGene x1 = null;
		PixelGene x2 = (PixelGene)chromosome.getGene(0);
		
		
		for(int i = 1; i < n; i++) {
			x1 = x2;
			x2 = (PixelGene)chromosome.getGene(i);
			result += x1.distance(x2);
		}
		//System.out.println("F(X): " + result);
		return result;
	}
	
	/**
	 * 
	 * @param chromosome
	 * @return
	 */
	public static long fitness(IChromosome chromosome) {
		//System.out.println("unscaled fitness: " + (getCMax(chromosome) - costFunction(chromosome)));
		return getCMax(chromosome) - costFunction(chromosome);
	}
	
	@Override
	protected double evaluate(IChromosome a_subject) {
		double a = f_max / (f_max - f_min);
		double b = f_max * f_min / (f_min - f_max);
		/*System.out.println("a: " + a);
		System.out.println("b: " + b);
			
		System.out.println("fitness: " + ((double)fitness(a_subject) / getCMax(a_subject))); */
		return (double)fitness(a_subject) / getCMax(a_subject);
	}
}
