package hr.fer.ga.imgcompression;
/**
 * 
 */
import org.jgap.BaseGene;
import org.jgap.Configuration;
import org.jgap.Gene;
import org.jgap.InvalidConfigurationException;
import org.jgap.RandomGenerator;
import org.jgap.UnsupportedRepresentationException;



/**
 * 
 * 
 * @author Ivan Kut
 */
public class PixelGene extends BaseGene {

	/** za serijalizaciju */
	private static final long serialVersionUID = 1L;

	/** alel gena */
	private PixelAllele allele;
	
	/**
	 * 
	 * 
	 * @param config
	 * @param allele
	 * @throws InvalidConfigurationException
	 */
	public PixelGene(final Configuration config, PixelAllele allele) 
				throws InvalidConfigurationException {
		super(config);
		setAllele(allele);
	}
	
	@Override
	public void applyMutation(int index, double a_percentage) {
		System.out.println("PixelGene.this.applyMutation(" + index + ", " + a_percentage + ")");		
	}
	
	@Override
	public int compareTo(Object o) {
		if(o instanceof PixelGene) {
			PixelGene other = (PixelGene)o;
			return this.allele.compareTo(other.allele);
		} else {
			return 1;
		}
	}
		
	@Override
	protected Object getInternalValue() {
		return allele;
	}
	
	@Override
	protected Gene newGeneInternal() {
		//System.out.println("newGeneInternal()");
		try {
			return new PixelGene(getConfiguration(), allele);
		} catch(InvalidConfigurationException e) {
			throw new IllegalStateException(e.getMessage());
		}
	}
	
	@Override
	public void setAllele(Object value) {
		if(value instanceof PixelAllele) {
			allele = (PixelAllele)value;
		} else {
			throw new IllegalArgumentException("Invalid allele type! This gene type can only contain PixelAlleles.");
		}
	}
	
	public void setAllele(PixelAllele value) {
		this.allele = value;
	}
	
	public PixelAllele getAllele() {
		return allele;
	}
	
	@Override
	public void setToRandomValue(RandomGenerator generator) {
		System.out.println("PixelGene.this.setToRandomValue(" + generator + ")");
	}
	
	@Override
	public void setValueFromPersistentRepresentation(String a_representation)
			throws UnsupportedOperationException,
			UnsupportedRepresentationException {
		
	}
	
	@Override
	public String getPersistentRepresentation()
			throws UnsupportedOperationException {

		return null;
	}
	
	/**
	 * Metoda koja rauna udaljenost dvaju pixela slike.
	 * Pixeli se zadaju kao <code>PixelAllele</code> objekti.
	 * Usporeuje bajtove koji predstavljaju boju piksela. 
	 * Boja svakog pixela predstavljena je <code>Color</code>
	 * objektom. 
	 * <ul>Usporeuje vrijednosti bajtova koji reprezentiraju:
	 * 	<li>crvenu boju</li>
	 * 	<li>zelenu boju</li>
	 * 	<li>plavu boju</li>
	 * 	<li>alpha channel</li>
	 * </li>
	 * Udaljenost se definira kao suma kvadrata razlike vrijednosti
	 * svakog od navedenih bajtova. 
	 * 
	 * @param x1	prvi piksel
	 * @param x2	drugi piksel
	 * @return		udaljenost piksela
	 */
	public int distance(PixelGene other) {
		int result = 0;
		int byteDifference;
		byteDifference = this.allele.getColor().getRed()
							- other.allele.getColor().getRed();
		result += byteDifference * byteDifference;
		byteDifference = this.allele.getColor().getGreen()
							- other.allele.getColor().getGreen();
		result += byteDifference * byteDifference;
		byteDifference = this.allele.getColor().getBlue() 
							- other.allele.getColor().getBlue();
		result += byteDifference * byteDifference;
		byteDifference = this.allele.getColor().getAlpha() 
							- other.allele.getColor().getAlpha();
		result += byteDifference * byteDifference;
		return result;
	}
}
